function [fpsi_n,fphi_n] = add_global_noise(ln,noise_type,fpsi,fphi,psi,phi)

  % Add noise to the modulus of the wavelet transform

  % Inputs:
  % ln : amount of noise (in relative l2-norm)
  % noise_type : type of noise
  %              ('simple_gaussian' or 'gaussian_by_frequency')
  % fpsi : modulus of the wavelet transform
  % fphi : low-frequency component (optional)
  % psi, phi : wavelets
	 
  % Outputs:
  % fpsi_n : modulus of the wavelet transform + noise
  % fphi : low-frequency component + noise

  if strcmp(noise_type,'simple_gaussian')
    % gaussian white noise

    % generate the noise
    noise_psi = randn(size(fpsi)) ;
    if (nargin>3) && ~isempty(fphi)
      noise_phi = (randn(size(fphi))+i*randn(size(fphi)))/sqrt(2) ;    
    end

  elseif strcmp(noise_type,'gaussian_by_frequency')
    % gaussian white noise, modulated so that low-frequency components
    % receive only low-frequency noise

    % compute low-pass filters whose bandwith is of the order of the
    % respective characteristic frequencies of the wavelets
    low_pass = sqrt(flipud(cumsum(abs(flipud(psi).^2)))) ;
    low_pass(:,1) = max(abs(low_pass(:))) ;
    % choose the noise; filter it by the low-pass filters
    noise_psi = randn(size(fpsi)).*low_pass ;
    noise_psi = real(ifft(noise_psi,[],2)) ;
    noise_psi = bsxfun(@rdivide,noise_psi, ...
		       sqrt(sum(abs(noise_psi).^2,2))) ;
    if (nargin>3) && ~isempty(fphi)
      flip_phi = [phi(1),fliplr(phi(2:end))] ;
      noise_phi = (randn(size(fphi))+i*randn(size(fphi))).*phi + ...
		  (randn(size(fphi))+i*randn(size(fphi))).*flip_phi ;
      noise_phi = ifft(noise_phi) ;
      noise_phi = noise_phi / norm(noise_phi,2) ;
    end

  else
      error('add_global_noise: unknown type of noise')
  end
    
  % compute the norms
  if (nargin>3) && ~isempty(fphi)
    noise_norm = sqrt(sum(abs(noise_psi(:)).^2)+sum(abs(noise_phi).^2)) ;
    transf_norm = sqrt(sum(abs(fpsi(:)).^2) + sum(abs(fphi).^2)) ;
  else
    noise_norm = sqrt(sum(abs(noise_psi(:)).^2)) ;
    transf_norm = sqrt(sum(abs(fpsi(:)).^2)) ;
  end  
  
  % add the noise
  fpsi_n = abs(fpsi+ln*transf_norm/noise_norm*noise_psi) ;
  if (nargout>1)
    if (nargin>3) && ~isempty(fphi)
      fphi_n = fphi + ln*transf_norm/noise_norm*noise_phi ;
    else
      error(['Function add_global_noise: the low-frequency component', ...
	     ' of the wavelet transform has not been given.'])
    end
  end
