function fpsi = eval_lowest(mod1,mod2,psi1,psi2,psi_low)
    
  % From |f star psi1| and |f star psi2|, reconstruct f star psi_low

  % It is assumed that psi1 and psi2 are low-frequency wavelets and
  % there support in the Fourier domain is small. The principle of the
  % algorithm is to find all the functions g such that |g star psi1| =
  % |f star psi1| and all the functions h such that |h star psi2| = |f
  % star psi2|. Then the correct f is the only function which is both
  % in the list of possible g's and in the list of possible h's

  % Inputs
  % mod1: |f star psi1|
  % mod2: |f star psi2|
  % psi1,psi2: low-frequency wavelets (in the Fourier domain)
  %            psi1 must be of higher characteristic frequency than
  %            psi2
  % psi_low: wavelet (or list of wavelets, each wavelet being a row)

  % Output
  % fpsi: f star psi_low

  N = size(psi1,2) ; % size of the signals

  sol1 = solutions(mod1,psi1,13) ; % possible g star psi1 (in Fourier)
  n1 = size(sol1,1) ; % number of possible g's
  sol2 = solutions(mod2,psi2,13) ; % possible h star psi2 (in Fourier)
  n2 = size(sol2,1) ; % number of possible h's

  % the possible g star psi1 and h star psi2 are only computed on the
  % approximate support of psi1 and psi2
  ind = min(size(sol1,2),size(sol2,2)) ; % size of the smallest of the
                                         % two supports

  ff1 = bsxfun(@rdivide,sol1(:,2:ind),psi1(2:ind)) ; % possible g's
  ff2 = bsxfun(@rdivide,sol2(:,2:ind),psi2(2:ind)) ; % possible h's

  scals = ff1 * ff2' ;
  mat_dist = repmat(sum(abs(ff1).^2,2),[1,n2]) + ...
	     repmat(sum(abs(ff2).^2,2),[1,n1])' - ...
	     2*abs(scals) ;
  % mat_dist(i,j) is the min over all z's such that |z|=1 of the
  % l2-distance between the i-th possible g and the j-th possible h
  % multiplied by z
  
  [forg,i_min] = min(min(mat_dist,[],1)) ;
  % sol2(i_min,:) is the correct solution

  % from this solution, compute f star psi_low
  nb_low = size(psi_low,1) ; % number of wavelets in psi_low
  ind = size(sol2,2) ;
  fpsi = bsxfun(@times,psi_low(:,2:ind),sol2(i_min,2:ind)./psi2(2:ind)) ;
  fpsi = ifft([zeros(nb_low,1),fpsi],N,2) ;
