function exp_decay = exp_decay_approx(psi,a,param)

  % Determines whether the wavelets decay exponentially or not;
  % if they do not, compute the indexes where they will be truncated

  % Inputs
  % psi: wavelets (each row is a wavelet)
  % a: dilation factor
  % param: 'yes' if the wavelet decrease exponentially
  %        'no' if they do not
  %        'auto' if this must be automatically determined

  % Ouput
  % exp_decay: structure with two fields
  %   -> exp_decay.bool: whether the decay is exponential
  %   -> exp_decay.cutoff: truncation indexes
  %                        (exists only if exp_decay.bool = false)

  if strcmp(param,'auto')

    % determines whether the wavelet decay exponentially

    % principle: find the largest M such that
    %    psi(j,om) <= 5 * exp(M * (a^(j-1)om - N/2) * 2/N)
    % the wavelets are said to decrease exponentially if M is large
    % enough
    
    nbf = size(psi,1) ;
    N = size(psi,2) ;

    log_psi = - log(abs(psi)/5+1e-13) ;
    ref = a.^[0:nbf-1]' * [0:N-1] * 2/N - 1 ;

    inds = find((ref>0).*(abs(psi)>1e-12)) ;
    min_log = min(log_psi(inds)./ref(inds)) ; % M

    if (min_log < log(10)/5)
      exp_decay.bool = false ;
      fprintf(['Warning: the wavelets do not seem to decrease ', ...
	       'exponentially.\n']) ;
    else
      exp_decay.bool = true ;
    end

  elseif strcmp(param,'yes')

    exp_decay.bool = true ;
    
  elseif strcmp(param,'no')
	 
    exp_decay.bool = false ;

  else

    error(['Function exp_decay_approx: the parameter exp_decay must be', ...
	   '"yes", "no" or "auto". Here, I received "', param, '".'])

  end

  if ~exp_decay.bool

    % if the decay is not exponential, compute, for each scale, the index
    % above which the wavelet is negligible and can be truncated
    % criterion for being negligible: the absolute value is less that 0.02
    % times the maximal absolute value

    for s=1:nbf
      is_small = (abs(psi(s,:)) < 0.02 * max(abs(psi(s,:)))) ;
      stays_small = fliplr(cumprod(fliplr(is_small))) ;

      if (max(stays_small)<1)
	cutoff(s) = N ;
      else
	[forg,i_jump] = max(stays_small) ;
	cutoff(s) = i_jump ;
      end
	 
    end

    exp_decay.cutoff = cutoff ;

  end
