function alpha = find_alpha(psi1,psi2,psi3,a,exp_decay,j0)

  % Inputs
  % psi1: wavelet of maximal frequency
  % psi2: wavelet of frequency just under psi1
  % psi3: wavelet of frequency just under psi2
  % a: dilation factor
  % exp_decay: whether the wavelets decrease exponentially or not
  % j0: index of psi1 (psi1 = psi(j0,:))

  % Output
  % alpha: real number such that psi1 .* exp(-[0:N-1] * alpha)
  %        appears to be optimal as an intermediate wavelet in the
  %        reconstruction process

  N = size(psi1,2) ; % size of the signals  

  % Dichotomy search
  nb_it = 5 ; % number of iterations
  prec = 6 ; % number of values tested at each iteration

  % initial bounds
  alpha_min = 0 ;
  [forg,i_max] = max(abs(psi1)) ;
  alpha_max = 1/i_max * log(50) ;
  % the maximal bound is such that exp(-[0:N-1]*alpha) is equal to
  % 0.02 at the characteristic frequency of psi1

  oms = [0:N-1] ;
  if exp_decay.bool
    oms2 = oms ;
  else
    % truncate at the cutoff frequency if the wavelets do not decrease
    % exponentially
    oms2 = exp_decay.cutoff(j0+1) - abs(exp_decay.cutoff(j0+1)-oms) ;
  end

  for k=1:nb_it

    % values to be tested
    alphas = alpha_min + [0:prec-1]/(prec-1) * (alpha_max-alpha_min) ;
      
    for l=1:prec

      psi1_m = abs(psi1 .* exp(-oms*alphas(l))) ;
      psi2_m = abs(psi2 .* exp(oms2*alphas(l)*a)) ;
      psi3_m = abs(psi3 .* exp(-oms*alphas(l)*a^2)) ;

      % psi1_m is the (absolute value of the) candidate for psi_low(1,:)
      % psi2_m is the (absolute value of the) candidate for psi_high(2,:)
      % psi3_m is a slightly arbitrary low-frequency function
      
      % In the reconstruction algorithm, (f star psi_low(1,:)) will be
      % estimated from (f star psi_high(2,:)), assuming something like
      % (f star psi3_m) is already known ; we must thus choose alpha so
      % that this estimation is the most stable possible.
      % We choose alpha so that the difference between psi_low(1,:)
      % (l2-normalized) and its projection onto the vector space generated
      % by psi_high(2,:) and psi3_m has the smallest possible norm

      % normalization and orthogonalization of (psi2_m,psi3_m)
      psi1_m = psi1_m / norm(psi1_m,2) ;
      psi2_m = psi2_m / norm(psi2_m,2) ;
      psi3_m = psi3_m - sum(psi2_m.*psi3_m)*psi2_m ;
      psi3_m = psi3_m / norm(psi3_m,2) ;

      % (squared) norm of the difference
      norm_diff(l) = 1 - sum(psi2_m.*psi1_m)^2 - sum(psi3_m.*psi1_m)^2 ;

    end

    [forg,i_min] = min(norm_diff) ;      

    i_inf = max(1,i_min-1) ;
    i_sup = min(prec,i_min+1) ;
      
    % new bounds
    alpha_min = alphas(i_inf) ;
    alpha_max = alphas(i_sup) ;

  end

  alpha = alphas(i_min) ;
