function f_rec = gerchberg_saxton(f_init,fpsi,psi,Kit,nb_print,N_ff)

  % This function attempts to reconstruct a function from the modulus
  % of its wavelet transform by the method of Gerchberg and Saxton
  % (alternate projections, described in 'A practical algorithm for the
  % determination of phase from image and diffraction plane pictures'
  % (Optik, 1972)
    
  % Inputs :
  % f_init : initial guess (optional)
  % fpsi : modulus of the wavelet transform
  % psi : wavelets
  % Kit : number of iterations (optional)
  % nb_print : print a message every nb_print iteration (optional)
  % N_ff : size of the support of f (in Fourier) (optional, by default
  %        half the length of the signal)

  % Output :
  % f_rec : reconstructed function

  N = size(fpsi,2) ; % size of the signal
 
  if (nargin<4) || isempty(Kit)
    Kit = 200 ;
  end
  if (nargin<5) || isempty(nb_print)
    nb_print = 500 ;
  end
  if (nargin<6) || isempty(N_ff)
    N_ff = floor(N/2)+1 ;
  end

  if (numel(f_init)==0)
    % if no initial guess has been provided, choose one at random
    ff = randn(1,N)+i*randn(1,N) ;
    ff(1) = 0 ;
    ff(N_ff+1:end) = 0 ;
    f_init = ifft(ff) ;
  end
    
  f_rec = f_init ;
  ff_rec = fft(f_rec) ;

  psi_conj = dual_wavelets(psi) ;
  psi_conj(:,[1,N_ff+1:end]) = 0 ;

  fprintf('%d iterations of the Gerchberg-Saxton algorithm (', ...
	  Kit) ;

  % iterations
  for k=1:Kit

    if (mod(k,nb_print)==0)
      fprintf('%d ',k) ;
    end
    
    % Project the wavelet transform of f_rec on the set of functions
    % whose moduli correspond to the desired ones
    psi_rec = ifft(bsxfun(@times,psi,ff_rec),[],2) ;
    psi_rec = fpsi.*psi_rec./(abs(psi_rec)+1e-13) ;
    
    % Project psi_rec on the set of admissible wavelet transforms
    psi_rec_f = fft(psi_rec,[],2) ;
    ff_rec = sum(psi_rec_f.*psi_conj,1) ;
    
  end

  f_rec = ifft(ff_rec) ;

  fprintf(')\n') ;
