function [fpsi_low,fpsi_high] = ...
	 gradient_descent(fpsi_low,fpsi_high,products,psi_low, ...
			  psi_high,min_freq,max_ind,opts)

  % This function iteratively optimizes the current values of
  % fpsi_high(s,:) for s>min_freq and fpsi_low(s,:) for s>=min_freq.
  % An estimation of the underlying signal f is also jointly
  % optimized.
  % It uses l-bfgs.

  % The objective function is 
  %   norm(inv_max .* diff_prod)^2 + al * norm(diff_ff)^2
  % where:
  %  - diff_prod is the difference between the desired products
  %              and fpsi_low .* conj(fpsi_high)
  %  - diff_ff is (something like) the difference between
  %            [fpsi_low,fpsi_high] and [f star psi_low,f star psi_high]
  %  - inv_max and al are positive normalization coefficients

  % During the estimation, for each s, fpsi_low(s,:) and
  % fpsi_high(s,:) are constrained to verify fpsi_low(s,:) star
  % psi_high(s,:) = fpsi_high(s,:) star psi_low(s,:)

  % Inputs:
  % fpsi_low, fpsi_high: current estimations of f star psi_low and
  %                        f star psi_high
  % products: values of fpsi_low .* conj(fpsi_high) computed from
  %             the moduli
  % psi_low, psi_high: auxiliary wavelets defined in reconstruct.m
  % min_freq: index of the highest-frequency wavelet that will be
  %           optimized
  % max_ind: size of the support of the highest frequency wavelet to
  %          be considered
  % opts: structure with two fields, Kit_descent (maximal number of
  %       iterations) and time_limit (maximal time devoted to the
  %       gradient descent)

  nbf = size(fpsi_low,1) ; % number of frequencies
  N = size(fpsi_low,2) ; % size of the signal

  nbf0 = nbf-min_freq ; % number of frequencies to be considered
 
  % extracts the lines of 'products', 'psi_low', 'psi_high' which
  % correspond to considered frequencies
  sub_prod = products(min_freq+1:nbf,:) ;
  sub_psi_low = psi_low(min_freq:nbf,:) ;
  sub_psi_high = psi_high(min_freq+1:nbf,:) ;

  % If the wavelets have a small support (in frequency), then all the
  % functions can be downsampled (to save time). We downsample each
  % signal so that it has N2 = min(N,3*max_ind) points.
  N2 = min(N,3*max_ind) ;
  % downsample sub_prod
  sub_prod_f = fft(sub_prod,[],2) ;
  sub_prod_f = [sub_prod_f(:,1:floor((N2+1)/2)), ...
		sub_prod_f(:,N-N2+floor((N2+1)/2)+1:N)] ;
  sub_prod = ifft(sub_prod_f,[],2) * N/N2 ;
    
  % normalization coefficients
  % components of the product with small norm must not be neglected
  % in the optimization: they are multiplied by a higher value than
  % the other ones
  inv_max = repmat(1./max(abs(sub_prod),[],2),[1,N2]) ;
  %inv_max = 1./abs(sub_prod) ;
  inv_max = min(inv_max,10*min(inv_max(:))) ;
  % precomputation of a term that will be used several times
  inv_max22N = (2/N2)*inv_max.^2 ;

  % for each s, we want fpsi_low(s-1,:) star psi_high(s,:) =
  %                     fpsi_high(s,:) star psi_low(s-1,:)
  % so we will look for fpsi_low(s-1,:) and fpsi_high(s,:) under the form
  %    fpsi_low(s-1,:) = g star mult_low(s-1,:)
  %    fpsi_high(s,:)  = g star mult_high(s,:)
  % where mult_low(s+1,:) and mult_high(s,:) are chosen such that
  %    mult_low(s+1,:) star psi_high(s,:)
  %       = mult_high(s,:) star psi_low(s+1,:)
  % we moreover impose abs(mult_low(s+1,:)) + abs(mult_high(s,:)) = 1 so
  % that g is well-conditioned from fpsi_low(s+1,:) and fpsi_high(s,:)
  mult = [abs(sub_psi_high) + ...
          abs(sub_psi_low(1:nbf0,:)) ; sub_psi_low(nbf0+1,:)] ;
  mult_low = [sub_psi_low(1:nbf0,:)./(mult(1:nbf0,:)+1e-15) ; ones(1,N)] ;
  mult_high = [sub_psi_high ./ (mult(1:nbf0,:)+1e-15)] ;

  mult = mult(:,1:max_ind) ;
  mult_low = mult_low(:,1:max_ind) ;
  mult_high = mult_high(:,1:max_ind) ;

  % initial value for g (in the Fourier domain)
  fpsi_low_f = fft(fpsi_low(min_freq:nbf,:),[],2) ;
  fpsi_high_f = fft(fpsi_high(min_freq+1:nbf,:),[],2) ;
  g = [fpsi_high_f(1:nbf0,1:max_ind).* ...
       exp(-i*angle(sub_psi_high(1:nbf0,1:max_ind))) + ...
       fpsi_low_f(1:nbf0,1:max_ind) ...
       .*exp(-i*angle(sub_psi_low(1:nbf0,1:max_ind))) ; ...
       fpsi_low_f(nbf0+1,1:max_ind)] ;

  % initial value for the Fourier transform of f
  div_tmp = sum(abs(sub_psi_low).^2,1) + sum(abs(sub_psi_high).^2,1) ;
  div_tmp = div_tmp(1:max_ind) ;
  div_tmp(1) = 1 ;
  ff = sum(fpsi_low_f.*conj(sub_psi_low),1) + ...
       sum(fpsi_high_f.*conj(sub_psi_high),1) ;
  ff = ff(1:max_ind) ./ div_tmp ;

  % value of the second normalization coefficient
  al =  0.5 * norm(sub_prod.*inv_max,'fro')^2 / norm(g(:),2)^2 ;

  % initialization of the loop
  step = 0 ;
  k_it = 0 ;
  NotConverged = true ;
  time_cur = 0 ;
  time_init = toc ;
  % data structures used for l-bfgs
  m_bfgs = 3 ;
  last_g = zeros(size(g,1),size(g,2),m_bfgs) ;
  last_ff = zeros(m_bfgs,size(ff,2)) ;
  last_grads_g = zeros(size(g,1),size(g,2),m_bfgs) ;
  last_grads_ff = zeros(m_bfgs,size(ff,2)) ;
  nb_stored = 0 ;

  fprintf ('Gradient descent (scale %d): ',min_freq) ;

  % beginning of the loop
  while (k_it<opts.Kit_descent) && (time_cur<opts.time_limit) ...
	&& NotConverged
    
    % increment counter
    k_it = k_it +1 ;
    
    % print info
    if (mod(k_it,opts.nb_print_descent)==0)
      fprintf('%d - ',k_it) ;
    end
    
    % difference between true and current products
    fpsi_low_tmp = ifft(g(2:nbf0+1,:).*mult_low(2:nbf0+1,:),N2,2) ;
    fpsi_high_tmp = ifft(g(1:nbf0,:).*mult_high,N2,2) ;
    diff_prod = fpsi_low_tmp.*conj(fpsi_high_tmp)-sub_prod ;

    % difference between the g(s,:) and the f star mult(s,:)
    % (g(s,:) = f star mult(s,:) for any s is equivalent to
    %  [fpsi_low,fpsi_high] = [f star psi_low,f star psi_high])
    diff_ff = bsxfun(@times,mult,ff) - g ;

    % current value of the objective        
    obj(k_it) = norm(inv_max.*diff_prod,'fro').^2 + ...
		al * sum(sum(abs(diff_ff).^2)) ;
 
    % if the objective has increased, go back to the previous values
    % and take a smaller step
    if (k_it>1) && (obj(k_it)>=obj(k_it-1))
       
      obj(k_it) = obj(k_it-1) ;
      g = prev_g ;
      ff = prev_ff ;

      step = step/2 ;
            
    else
         
      % store g and ff
      prev_g = g ;
      prev_ff = ff ;
            
      % compute the gradient

      % gradient along g
      % initialize
      grad_g = zeros(nbf0+1,max_ind) ;
      % first term of the objective
      grad_tmp = fft(fpsi_high_tmp.*diff_prod ...
                     .* inv_max22N,[],2) ;
      grad_g(2:nbf0+1,:) = ...
      grad_tmp(:,1:max_ind).*conj(mult_low(2:nbf0+1,:)) ;                
      grad_tmp = fft(fpsi_low_tmp.*conj(diff_prod ...
					.* inv_max22N),[],2) ;
      grad_g(1:nbf0,:) = grad_g(1:nbf0,:) + ...
			 grad_tmp(:,1:max_ind).*conj(mult_high) ;
      % second term
      grad_g = grad_g - 2 * al * diff_ff ;

      % gradient along ff
      grad_ff = 2 * al * sum(diff_ff.*conj(mult),1) ;               

      % store the values
      if (norm(grad_g-last_grads_g(:,:,1),2)>1e-4 * norm(grad_g,2))
	last_g = circshift(last_g,[0,0,1]) ;
	last_g(:,:,1) = g ;
	last_ff = circshift(last_ff,[1,0]) ;
	last_ff(1,:) = ff ;
	last_grads_g = circshift(last_grads_g,[0,0,1]) ;
	last_grads_g(:,:,1) = grad_g ;
	last_grads_ff = circshift(last_grads_ff,[1,0]) ;
	last_grads_ff(1,:) = grad_ff ;
	nb_stored = nb_stored + 1 ;
      end
    end

    if (mod(k_it,1000)==1)

      % compute an approximation of the hessian matrix, in order to
      % speed up convergence      
      % this computation is very costly so it cannot be performed at
      % each iteration step

      % recompute some quantities
      fpsi_low_tmp = ifft(g(2:nbf0+1,:).*mult_low(2:nbf0+1,:),N2,2) ;
      fpsi_high_tmp = ifft(g(1:nbf0,:).*mult_high,N2,2) ;
      
      % only the diagonal of the hessian is computed
      % component along g
      hess_g = zeros(nbf0+1,max_ind) ;
      % first part of the objective
      h_tmp = bsxfun(@times, abs(mult_low(2:nbf0+1,:)).^2, ...
		     sum((abs(fpsi_high_tmp).*inv_max).^2,2)) * 2 / N2^2 ;
      hess_g(2:(nbf0+1),:) = h_tmp ;
      h_tmp = bsxfun(@times,abs(mult_high).^2, ...
		     sum((abs(fpsi_low_tmp).*inv_max).^2,2)) * 2 / N2^2 ;
      hess_g(1:nbf0,:) = hess_g(1:nbf0,:) + h_tmp ;
      % second part of the objective
      hess_g = 2*al + hess_g ;
      
      % component along ff
      hess_ff = 2*al * sum(abs(mult).^2,1) ;
      hess_ff(find(abs(hess_ff)<1e-12)) = 1 ;

      % inversion
      h_inv_g = hess_g.^(-1) ;
      h_inv_ff = hess_ff.^(-1) ;
                
    end

    % l-bfgs: refine the approximation of the inverse hessian
    % copy the gradient
    grad_g2 = grad_g ;
    grad_ff2 = grad_ff ;
    if (nb_stored>=m_bfgs)

      last_g_diff = last_g(:,:,1:end-1) - last_g(:,:,2:end) ;
      last_ff_diff = last_ff(1:end-1,:) - last_ff(2:end,:) ;
      last_grads_g_diff = last_grads_g(:,:,1:end-1) - ...
			  last_grads_g(:,:,2:end) ;
      last_grads_ff_diff = last_grads_ff(1:end-1,:) - ...
			   last_grads_ff(2:end,:) ;

      r = real(sum(sum(conj(last_g_diff).*last_grads_g_diff,1),2)) ;
      r = r(:) + real(sum(conj(last_ff_diff).*last_grads_ff_diff,2)) ;

      % first loop
      for ii=1:m_bfgs-1
	if (r(ii)>1e-8)
	  alpha(ii) = (real(sum(sum(conj(last_g_diff(:,:,ii)) ...
				    .* grad_g2))) +  ...
		       real(sum(conj(last_ff_diff(ii,:)) ...
				.*grad_ff2))) / r(ii) ;
	else
	  alpha(ii) = 0 ;
	end
	grad_g2 = grad_g2 - alpha(ii) * last_grads_g_diff(:,:,ii) ;
	grad_ff2 = grad_ff2 - alpha(ii) * last_grads_ff_diff(ii,:) ;
      end

      % multiplication by the first approximation of the
      % inverse hessian
      grad_g2 = grad_g2.*h_inv_g ;
      grad_ff2 = grad_ff2.*h_inv_ff ;

      % second loop
      for ii=(m_bfgs-1):(-1):1
	if (r(ii)>1e-8)
	  be(ii) = (real(sum(sum(conj(last_grads_g_diff(:,:,ii)) ...
				 .* grad_g2))) +  ...
		  real(sum(conj(last_grads_ff_diff(ii,:)) ...
			   .*grad_ff2))) / r(ii) ;
	else
	  be(ii) = 0 ;
	end
	grad_g2 = grad_g2 + (alpha(ii)-be(ii)) * last_g_diff(:,:,ii) ;
	grad_ff2 = grad_ff2 + (alpha(ii)-be(ii)) * last_ff_diff(ii,:) ;
      end
    end

    if (mod(k_it,50)==1)
      % initialisation of step
      % after 50 iterations, step is set back to a large value
      step = 0.1 * min(norm(g(:),2)/max(norm(grad_g2(:),2),1e-12), ...
                       norm(ff(:),2)/max(norm(grad_ff2(:),2),1e-12)) ;
    end

    % descent
    g = g - step * grad_g2 ;
    ff = ff - step * grad_ff2 ;

    % stopping criterium    
    if (k_it>40) && (obj(k_it)>opts.stop_crit*obj(floor(k_it/2)))
      NotConverged = 0 ;
    end
        
    time_cur = toc - time_init ;
    
  end
  fprintf('\n%d iterations\n',k_it) ;    
  
  % compute the values of fpsi_high and fpsi_low given by g
  fpsi_high(min_freq+1:end,:) = ifft(prev_g(1:nbf0,:).*mult_high,N,2) ;
  fpsi_low(min_freq:end,:) = ifft(prev_g.*mult_low,N,2) ;
